@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-file-helpers.h,v 1.3 2006/01/19 15:12:55 nshmyrev Exp $
+ *  $Id: ev-file-helpers.h,v 1.4 2006/11/15 11:33:09 carlosgc Exp $
  */
 
 #ifndef EPHY_FILE_HELPERS_H
@@ -25,17 +25,19 @@
 
 G_BEGIN_DECLS
 
-const char *ev_dot_dir               (void);
+const gchar *ev_dot_dir               (void);
 
-void        ev_file_helpers_init     (void);
+const gchar *ev_tmp_dir               (void);
 
-void        ev_file_helpers_shutdown (void);
+void         ev_file_helpers_init     (void);
 
-gchar*      ev_tmp_filename          (void);
+void         ev_file_helpers_shutdown (void);
 
-gboolean   ev_xfer_uri_simple        (const char *from,
-    				      const char *to,
-				      GError     **error);
+gchar*       ev_tmp_filename          (void);
+
+gboolean     ev_xfer_uri_simple       (const char *from,
+				       const char *to,
+				       GError     **error);
 
 G_END_DECLS
 
