@@ -46,7 +46,7 @@
 #include "ps-document.h"
 #include "ev-debug.h"
 #include "gsdefaults.h"
-#include "ev-ps-exporter.h"
+#include "ev-file-exporter.h"
 #include "ev-async-renderer.h"
 
 #define MAX_BUFSIZE 1024
@@ -70,30 +70,30 @@ struct record_list
 static gboolean broken_pipe = FALSE;
 
 /* Forward declarations */
-static void	ps_document_init			(PSDocument           *gs);
-static void	ps_document_class_init			(PSDocumentClass      *klass);
-static void	send_ps					(PSDocument           *gs,
-							 long                  begin,
-							 unsigned int          len,
-							 gboolean	       close);
-static void	output					(gpointer              data,
-							 gint                  source,
-							 GdkInputCondition     condition);
-static void	input					(gpointer              data,
-							 gint		       source,
-							 GdkInputCondition     condition);
-static void	stop_interpreter			(PSDocument           *gs);
-static gint	start_interpreter			(PSDocument	      *gs);
-static void	ps_document_document_iface_init		(EvDocumentIface      *iface);
-static void	ps_document_ps_exporter_iface_init	(EvPSExporterIface    *iface);
-static void	ps_async_renderer_iface_init		(EvAsyncRendererIface *iface);
+static void	ps_document_init			(PSDocument             *gs);
+static void	ps_document_class_init			(PSDocumentClass        *klass);
+static void	send_ps					(PSDocument             *gs,
+							 long                    begin,
+							 unsigned int            len,
+							 gboolean	         close);
+static void	output					(gpointer                data,
+							 gint                    source,
+							 GdkInputCondition       condition);
+static void	input					(gpointer                data,
+							 gint		         source,
+							 GdkInputCondition       condition);
+static void	stop_interpreter			(PSDocument             *gs);
+static gint	start_interpreter			(PSDocument	        *gs);
+static void	ps_document_document_iface_init		(EvDocumentIface        *iface);
+static void	ps_document_file_exporter_iface_init	(EvFileExporterIface    *iface);
+static void	ps_async_renderer_iface_init		(EvAsyncRendererIface   *iface);
 
 G_DEFINE_TYPE_WITH_CODE (PSDocument, ps_document, G_TYPE_OBJECT,
                          {
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT,
 							ps_document_document_iface_init);
-				 G_IMPLEMENT_INTERFACE (EV_TYPE_PS_EXPORTER,
-							ps_document_ps_exporter_iface_init);
+				 G_IMPLEMENT_INTERFACE (EV_TYPE_FILE_EXPORTER,
+							ps_document_file_exporter_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_ASYNC_RENDERER,
 							ps_async_renderer_iface_init);
 			 });
@@ -294,18 +294,6 @@ send_ps (PSDocument *gs, long begin, unsigned int len, gboolean close)
 	}
 }
 
-static float
-get_xdpi (PSDocument *gs)
-{
-	return 25.4 * gdk_screen_width() / gdk_screen_width_mm();
-}
-
-static float
-get_ydpi (PSDocument *gs)
-{
-	return 25.4 * gdk_screen_height() / gdk_screen_height_mm();
-}
-
 static void
 setup_pixmap (PSDocument *gs, int page, double scale, int rotation)
 {
@@ -447,19 +435,17 @@ static void
 setup_page (PSDocument *gs, int page, double scale, int rotation)
 {
 	gchar *buf;
-	char scaled_xdpi[G_ASCII_DTOSTR_BUF_SIZE];	
-	char scaled_ydpi[G_ASCII_DTOSTR_BUF_SIZE];
+	char scaled_dpi[G_ASCII_DTOSTR_BUF_SIZE];	
 	int urx, ury, llx, lly;
 
 	LOG ("Setup the page");
 
 	get_page_box (gs, page, &urx, &ury, &llx, &lly);
-	g_ascii_dtostr (scaled_xdpi, G_ASCII_DTOSTR_BUF_SIZE, get_xdpi (gs) * scale);
-	g_ascii_dtostr (scaled_ydpi, G_ASCII_DTOSTR_BUF_SIZE, get_ydpi (gs) * scale);
+	g_ascii_dtostr (scaled_dpi, G_ASCII_DTOSTR_BUF_SIZE, 72.0 * scale);
 
 	buf = g_strdup_printf ("%ld %d %d %d %d %d %s %s %d %d %d %d",
 			       0L, rotation, llx, lly, urx, ury,
-			       scaled_xdpi, scaled_ydpi,
+			       scaled_dpi, scaled_dpi,
 			       0, 0, 0, 0);
 	LOG ("GS property %s", buf);
 
@@ -624,6 +610,7 @@ start_interpreter (PSDocument *gs)
 
 	char *argv[NUM_ARGS], *dir, *gv_env, *gs_path;
 	char **gs_args, **alpha_args = NULL;
+	char **gv_env_vars = NULL;
 	int argc = 0, i;
 
 	LOG ("Start the interpreter");
@@ -679,9 +666,10 @@ start_interpreter (PSDocument *gs)
 		return -1;
 	}
 
-	gv_env = g_strdup_printf ("GHOSTVIEW=%ld %ld",
+	gv_env = g_strdup_printf ("GHOSTVIEW=%ld %ld;DISPLAY=%s",
                            	  gdk_x11_drawable_get_xid (gs->pstarget),
-				  gdk_x11_drawable_get_xid (gs->bpixmap));
+				  gdk_x11_drawable_get_xid (gs->bpixmap),
+				  gdk_display_get_name (gdk_drawable_get_display (gs->pstarget)));
 	LOG ("Launching ghostview with env %s", gv_env);
 
 	gs->interpreter_pid = fork ();
@@ -717,7 +705,11 @@ start_interpreter (PSDocument *gs)
 				}
 			}
 
-			putenv(gv_env);
+			gv_env_vars = g_strsplit (gv_env, ";", -1);
+			g_free (gv_env);
+			for (i = 0; gv_env_vars[i]; i++) {
+				putenv (gv_env_vars[i]);
+			}
 
 			/* change to directory where the input file is. This helps
 			 * with postscript-files which include other files using
@@ -731,8 +723,8 @@ start_interpreter (PSDocument *gs)
 			/* Notify error */
 			g_critical ("Unable to execute [%s]\n", argv[0]);
 			g_strfreev (gs_args);
-			g_free (gv_env);
 			g_strfreev (alpha_args);
+			g_strfreev (gv_env_vars);
 			_exit (1);
 			break;
 		default:                     /* parent */
@@ -897,7 +889,7 @@ check_filecompressed (PSDocument * gs)
 
 		/* report error */
 		filename_dsp = g_filename_display_name (gs->gs_filename);
-		msg = g_strdup_printf (_("Error while decompressing file %s:\n"), filename_dsp);
+		msg = g_strdup_printf (_("Error while decompressing file “%s”:\n"), filename_dsp);
 		g_free (filename_dsp);
 		
 		interpreter_failed (gs, msg);
@@ -957,7 +949,7 @@ document_load (PSDocument *gs, const gchar *fname)
 			gchar *msg;
 
 			filename_dsp = g_filename_display_name (fname);
-			msg = g_strdup_printf (_("Cannot open file %s.\n"), filename_dsp);
+			msg = g_strdup_printf (_("Cannot open file “%s”.\n"), filename_dsp);
 			g_free (filename_dsp);
 			
 			interpreter_failed (gs, msg);
@@ -990,7 +982,10 @@ document_load (PSDocument *gs, const gchar *fname)
 static gboolean
 ps_document_next_page (PSDocument *gs)
 {
-	XEvent event;
+	XEvent      event;
+	GdkScreen  *screen;
+	GdkDisplay *display;
+	Display    *dpy;
 
 	LOG ("Make ghostscript render next page");
 
@@ -1000,14 +995,20 @@ ps_document_next_page (PSDocument *gs)
 
 	gs->busy = TRUE;
 
+	screen = gtk_window_get_screen (GTK_WINDOW (gs->target_window));
+	display = gdk_screen_get_display (screen);
+	dpy = gdk_x11_display_get_xdisplay (display);
+
 	event.xclient.type = ClientMessage;
-	event.xclient.display = gdk_display;
+	event.xclient.display = dpy;
 	event.xclient.window = gs->message_window;
-	event.xclient.message_type = gdk_x11_atom_to_xatom(gs_class->next_atom);
+	event.xclient.message_type =
+		gdk_x11_atom_to_xatom_for_display (display,
+						   gs_class->next_atom);
 	event.xclient.format = 32;
 
 	gdk_error_trap_push ();
-	XSendEvent (gdk_display, gs->message_window, FALSE, 0, &event);
+	XSendEvent (dpy, gs->message_window, FALSE, 0, &event);
 	gdk_flush ();
 	gdk_error_trap_pop ();
 
@@ -1079,7 +1080,7 @@ ps_document_load (EvDocument  *document,
 		    g_set_error(error,
 				G_FILE_ERROR,
 				G_FILE_ERROR_NOENT,
-				_("Failed to load document '%s'. Ghostscript interpreter was not found in path"),
+				_("Failed to load document “%s”. Ghostscript interpreter was not found in path"),
 				filename);
 		    g_free (filename_dsp);
 		    result = FALSE;	
@@ -1091,7 +1092,7 @@ ps_document_load (EvDocument  *document,
 			
 			g_set_error (error, G_FILE_ERROR,
 				     G_FILE_ERROR_FAILED,
-				     _("Failed to load document '%s'"),
+				     _("Failed to load document “%s”"),
 				     filename_dsp);
 			g_free (filename_dsp);
 		}
@@ -1212,14 +1213,14 @@ ps_document_get_page_size (EvDocument   *document,
 	PSDocument *gs = PS_DOCUMENT (document);
 	int urx, ury, llx, lly;
 
-	get_page_box (PS_DOCUMENT (document), page, &urx, &ury, &llx, &lly);
+	get_page_box (gs, page, &urx, &ury, &llx, &lly);
 
 	if (width) {
-		*width = (urx - llx) / 72.0 * get_xdpi (gs) + 0.5;
+		*width = (urx - llx) + 0.5;
 	}
 
 	if (height) {
-		*height = (ury - lly) / 72.0 * get_ydpi (gs) + 0.5;
+		*height = (ury - lly) + 0.5;
 	}
 }
 
@@ -1257,17 +1258,25 @@ ps_document_get_info (EvDocument *document)
 {
 	EvDocumentInfo *info;
 	PSDocument *ps = PS_DOCUMENT (document);
+	int urx, ury, llx, lly;
 
 	info = g_new0 (EvDocumentInfo, 1);
 	info->fields_mask = EV_DOCUMENT_INFO_TITLE |
 	                    EV_DOCUMENT_INFO_FORMAT |
 			    EV_DOCUMENT_INFO_CREATOR |
-			    EV_DOCUMENT_INFO_N_PAGES;
+			    EV_DOCUMENT_INFO_N_PAGES |
+	                    EV_DOCUMENT_INFO_PAPER_SIZE;
+
 	info->title = g_strdup (ps->doc->title);
 	info->format = ps->doc->epsf ? g_strdup (_("Encapsulated PostScript"))
 		                     : g_strdup (_("PostScript"));
 	info->creator = g_strdup (ps->doc->creator);
 	info->n_pages = ev_document_get_n_pages (document);
+	
+	get_page_box (PS_DOCUMENT (document), 0, &urx, &ury, &llx, &lly);
+
+	info->paper_width  = (urx - llx) / 72.0f * 25.4f;
+	info->paper_height = (ury - lly) / 72.0f * 25.4f;
 
 	return info;
 }
@@ -1289,10 +1298,22 @@ ps_async_renderer_iface_init (EvAsyncRendererIface *iface)
 	iface->render_pixbuf = ps_async_renderer_render_pixbuf;
 }
 
+static gboolean
+ps_document_file_exporter_format_supported (EvFileExporter      *exporter,
+					    EvFileExporterFormat format)
+{
+	return (format == EV_FILE_FORMAT_PS);
+}
+
 static void
-ps_document_ps_export_begin (EvPSExporter *exporter, const char *filename,
-			     int first_page, int last_page,
-                             double width, double height, gboolean duplex)
+ps_document_file_exporter_begin (EvFileExporter      *exporter,
+				 EvFileExporterFormat format,
+				 const char          *filename,
+				 int                  first_page,
+				 int                  last_page,
+				 double               width,
+				 double               height,
+				 gboolean             duplex)
 {
 	PSDocument *document = PS_DOCUMENT (exporter);
 
@@ -1306,7 +1327,7 @@ ps_document_ps_export_begin (EvPSExporter *exporter, const char *filename,
 }
 
 static void
-ps_document_ps_export_do_page (EvPSExporter *exporter, EvRenderContext *rc)
+ps_document_file_exporter_do_page (EvFileExporter *exporter, EvRenderContext *rc)
 {
 	PSDocument *document = PS_DOCUMENT (exporter);
 	
@@ -1316,7 +1337,7 @@ ps_document_ps_export_do_page (EvPSExporter *exporter, EvRenderContext *rc)
 }
 
 static void
-ps_document_ps_export_end (EvPSExporter *exporter)
+ps_document_file_exporter_end (EvFileExporter *exporter)
 {
 	PSDocument *document = PS_DOCUMENT (exporter);
 
@@ -1333,9 +1354,10 @@ ps_document_ps_export_end (EvPSExporter *exporter)
 }
 
 static void
-ps_document_ps_exporter_iface_init (EvPSExporterIface *iface)
+ps_document_file_exporter_iface_init (EvFileExporterIface *iface)
 {
-	iface->begin = ps_document_ps_export_begin;
-	iface->do_page = ps_document_ps_export_do_page;
-	iface->end = ps_document_ps_export_end;
+	iface->format_supported = ps_document_file_exporter_format_supported;
+	iface->begin = ps_document_file_exporter_begin;
+	iface->do_page = ps_document_file_exporter_do_page;
+	iface->end = ps_document_file_exporter_end;
 }
